constant thread_safe = 1;
#include <module.h>;
inherit "module";
inherit "roxenlib";

constant PRODNAME = "JSON-RPC";
constant module_type = MODULE_LOCATION|MODULE_PROVIDER;
constant module_name = PRODNAME + ": Bridge";
constant module_doc = "Wraps pike objects into json objects, accesible through json-rpc in a location in the virtual file system. " +
 "Objects are either global, or per-session. Global objects are added by the function put(). " + 
 "If a global object is not found, it looks in the session variable \"jsonrpc_objects\", which it " + 
 "expects to be a mapping of the type (string:Public.Parser.JSON.Wrapper) where the string is the requested object name.";

void create () {
  defvar("location", Variable.Location("/json-rpc/", 0, "Mountpoint", 
				       "Place in the virtual filesystem where the objects will be accessed from. " ));
}

mapping objects = ([]);

multiset query_provides()
{
  return (< "JSON-RPC" >);
}

void put (string name, object o) {
  objects[name] = Public.Parser.JSON.Wrapper(o);
}

int(0..1) is_valid (string index, object o) {
  return functionp(o[index]) && index != "create" && index != "destroy";
}

string js_obj (object o, string url, string objname) {
  string objdef =objname + " = {\n" +
    "  url : \"" + url + "\",\n" +
    map(indices(o), lambda(string n) {
      if (is_valid(n, o))
        return "  " + n + " : function(cb) { remoteRequest(this.url, cb, new Request(\"" + n + "\", arguments)) }";
    }) * ",\n" + "\n}";
  return objdef;
}

mapping find_file (string pathname, RequestID id) {
  object o;
  array path = pathname / ".";
  string index = path[0];

  if (zero_type(o = objects[index])) {
    mapping session = cache.get_session_data(id->cookies->RoxenUserID);
    if (!session || zero_type(session->jsonrpc_objects) || !mappingp(session->jsonrpc_objects))
      return 0;
    if (zero_type(o = session->jsonrpc_objects[index]))
      return 0;
  }

  if (object_program(o) != Public.Parser.JSON.Wrapper) {
    return Roxen.http_low_answer(404, "Object found but class is not Public.Parser.JSON.Wrapper");
  }

  if(sizeof(path) > 1) {
    if(path[1] == "js")
      return Roxen.http_string_answer(js_obj(o->o, query("location") + index, index), "application/x-javascript");
  }

  if (id->method != "POST")
    return Roxen.http_low_answer(405, pathname + ": Method must be POST");

  mapping request;
  if (id->data) {
    request = Public.Parser.JSON.parse(id->data);
  } else
    return Roxen.http_low_answer(405, "Missing request data");
  mapping response = o->call(request);
  return Roxen.http_string_answer(Public.Parser.JSON.serialize(response), "text/json");
}

array(string)|void find_dir (string p, RequestID id) {
  array list = indices(objects);
  mapping session = cache.get_session_data(id->cookies->RoxenUserID);
  if (!session || zero_type(session->jsonrpc_objects) || !mappingp(session->jsonrpc_objects))
    list += indices(session->jsonrpc_objects);
  return list;
}
