
/* example:

  Either include this:

    <script langauge="javascript" type="text/javascript" src="/json-rpc/server.js"></script>

  or do this:

    var server = ServerProxy("url");
    server.init();

  Server is then an object with methods that each take at least one argument. All but the
  first arguments are passed to the server, but the first argument needs to be a callback function:

    server.list_methods(callback);

  ... such as this one. The callback is called with the result from from the server:

    function callback(result) {
    
    }

*/

function ServerProxy (url) {
  this.url = url;
  this.methods = null;
}

ServerProxy.prototype.init = function (cb) {
  var request = new Request("list_methods");
  remoteRequest(this.url, function(m) { server.prepareMethods(m); eval(cb); }, request);
}

ServerProxy.prototype.prepareMethods = function (methods) {
  for(var i=0; i < methods.length; i++) {
    var method = methods[i];
    this[method] = new Function ("cb", "remoteRequest(this.url, cb, new Request(\"" + method + "\", arguments));");
  }
}

function Request(method, args) {
  this.method = method;
  if (args && args.length > 1) {
    this.params = new Array();
    for(var i=1; i<args.length; i++)
      this.params.push(args[i]);
  } else
    this.params = null;
  this.id = null;
}

var requestQueue;

function ReqObj() {
  this.request = null;
  this.url = null;
  this.fun = null;
}

ReqObj.prototype.stateChange = function() {
  if(this.request.readyState == 4) {
    if(this.request.status == 200) {
      if(this.request.getResponseHeader("content-type") == "text/json") {
        var response;
        eval("response = " + this.request.responseText);
        if(!response.error)
          this.fun(response.result);
        else
          throw (response.error);
      } else {
        alert("Weird response:\n" + this.request.responseText);
      }
    } else {
      alert("There was a problem retrieving remote object data:\n(" + this.url + ")\n" + this.request.statusText);
    }
    requestQueue.push(this);
  }
}

function remoteRequest(url, fun, data) {
  var newReq;

  if(!requestQueue)
    requestQueue = new Array();
  if(requestQueue.length > 0) {
    newReq = requestQueue.shift();
  } else {
    newReq = new ReqObj();
    if(window.XMLHttpRequest)
      newReq.request = new XMLHttpRequest();
    else if(window.ActiveXObject)
      newReq.request = new ActiveXObject("Microsoft.XMLHTTP");
  }

  newReq.fun = fun;
  newReq.url = url;
  newReq.request.onreadystatechange = function () { newReq.stateChange(); };

  var postContent = stringify(data);

  if(data) {
    newReq.request.open("POST", url, true);
    newReq.request.send(postContent);
  } else {
    newReq.request.open("GET", url, true);
    newReq.request.send(null);
  }
}

function isoDateTime(d) {
  return d.getFullYear() + "-" + (zpad(d.getMonth() + 1)) + "-" + zpad(d.getDate()) + " " + zpad(d.getHours()) + ":" + zpad(d.getMinutes()) + ":" + zpad(d.getSe\conds());
}

function stringify(arg) {
  var c, i, l, s = '', v;

  switch (typeof arg) {
  case 'object':
    if (arg) {
      if (arg.constructor == Array) {
        for (i = 0; i < arg.length; ++i) {
          v = this.stringify(arg[i]);
          if (s) {
            s += ',';
          }
          s += v;
        }
        return '[' + s + ']';
      } else if (arg.constructor == Date) {
        return "\"" + isoDateTime(arg) + "\"";
      } else if (typeof arg.toString != 'undefined') {
        for (i in arg) {
          v = arg[i];
          if (typeof v != 'undefined' && typeof v != 'function') {
            v = this.stringify(v);
            if (s) {
              s += ',';
            }
            s += this.stringify(i) + ':' + v;
          }
        }
        return '{' + s + '}';
      }
    }
    return 'null';
  case 'number':
    return isFinite(arg) ? String(arg) : 'null';
  case 'string':
    l = arg.length;
    s = '"';
    for (i = 0; i < l; i += 1) {
      c = arg.charAt(i);
      if (c >= ' ') {
        if (c == '\\' || c == '"') {
          s += '\\';
        }
        s += c;
      } else {
        switch (c) {
        case '\b':
          s += '\\b';
          break;
        case '\f':
          s += '\\f';
          break;
        case '\n':
         s += '\\n';
          break;
        case '\r':
          s += '\\r';
          break;
        case '\t':
          s += '\\t';
          break;
        default:
          c = c.charCodeAt();
          s += '\\u00' + Math.floor(c / 16).toString(16) +
            (c % 16).toString(16);
        }
      }
    }
    return s + '"';
  case 'boolean':
    return String(arg);
  default:
    return 'null';
  }
}
